package com.gydx.component;

import java.io.IOException;
import java.util.concurrent.CopyOnWriteArraySet;
import javax.websocket.OnClose;
import javax.websocket.OnError;
import javax.websocket.OnMessage;
import javax.websocket.OnOpen;
import javax.websocket.Session;
import javax.websocket.server.PathParam;
import javax.websocket.server.ServerEndpoint;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.stereotype.Component;

/**
 *  --------------------- 竞价信息 ---------------------
 *  1.进入页面直接建立socket链接
 *  2.查找所有竞价单号，对应状态为进行中的，依据点击时间判断倒计时
 *  3.将倒计时变更的信息更新到页面，直到链接关闭或者倒计时变为
 */
@ServerEndpoint(value = "/client/BiddingMes/{userId}")
@Component
public class WSBiddingMes {
    private static Logger logger = LogManager.getLogger(WSBiddingMes.class.getName());

    /**
     * 静态变量，用来记录当前在线连接数。应该把它设计成线程安全的。
     */
    private static int onlineCount = 0;

    /**
     * concurrent包的线程安全Set，用来存放每个客户端对应的MyWebSocket对象。
     */
    private static CopyOnWriteArraySet<WSBiddingMes> wsClientMap = new CopyOnWriteArraySet<>();

    /**
     * 与某个客户端的连接会话，需要通过它来给客户端发送数据
     */
    private Session session;

    /**
     * 连接建立成功调用的方法
     *
     * @param session 当前会话session
     */
    @OnOpen
    public void onOpen(@PathParam("userId") String userId, Session session) {
        this.session = session;
        wsClientMap.add(this);
        addOnlineCount();
        logger.info(session.getId() + "有新链接加入，当前链接数为：" + wsClientMap.size());
        new Thread(() -> {
            int x = 100;
            while (x > 0) {
                try {
                    sendMessage(String.format("当前的值为：%d", x));
                    Thread.sleep(1000);
                } catch (Exception e) {
                    logger.error("倒计时错误", e);
                }
                --x;
            }
        }).start();
    }

    /**
     * 连接关闭
     */
    @OnClose
    public void onClose() {
        wsClientMap.remove(this);
        subOnlineCount();
        logger.info("有一链接关闭，当前链接数为：" + wsClientMap.size());
    }

    /**
     * 收到客户端消息
     *
     * @param message 客户端发送过来的消息
     * @param session 当前会话session
     */
    @OnMessage
    public void onMessage(String message, Session session) throws IOException, InterruptedException {
        logger.info("来终端的警情消息:" + message);
    }

    /**
     * 发生错误
     */
    @OnError
    public void onError(Session session, Throwable error) {
        logger.info("wsClientMap发生错误!");
        error.printStackTrace();
    }

    /**
     * 给所有客户端群发消息
     *
     * @param message 消息内容
     */
    public void sendMsgToAll(String message) throws IOException {
        for (WSBiddingMes item : wsClientMap) {
            item.session.getBasicRemote().sendText(message);
        }
        logger.info("成功群送一条消息:" + wsClientMap.size());
    }

    public void sendMessage(String message) throws IOException {
        this.session.getBasicRemote().sendText(message);
        logger.info("成功发送一条消息:" + message);
    }

    public static synchronized int getOnlineCount() {
        return WSBiddingMes.onlineCount;
    }

    public static synchronized void addOnlineCount() {
        WSBiddingMes.onlineCount++;
    }

    public static synchronized void subOnlineCount() {
        WSBiddingMes.onlineCount--;
    }
}
