package com.gydx.demo;

import com.gydx.vo.ExpertMesVO;

import java.util.*;

public class Z_ExpertAssign {

    public List<Map<String, Object>> assignExperts(List<ExpertMesVO> experts) {
        List<Map<String, Object>> assignedExperts = new ArrayList<>();
        List<Set<String>> previousRounds = new ArrayList<>(); // 用于记录每轮的分组情况

        for (int i = 0; i < 6; i++) { // 分成六轮
            Collections.shuffle(experts); // 每轮打乱顺序
            List<List<ExpertMesVO>> currentRound = new ArrayList<>();
            Set<String> currentPairs = new HashSet<>(); // 记录本轮的配对情况

            int index = 0;
            while (index < experts.size()) {
                ExpertMesVO expert1 = experts.get(index);
                ExpertMesVO expert2 = experts.get(index + 1);

                if (expert1.getExpertType().equals("B") && expert2.getExpertType().equals("B")) {
                    Collections.shuffle(experts);
                    index = 0;
                    currentRound.clear();
                    currentPairs.clear();
                    continue;
                }

                String pairKey = expert1.getExpertID() + "-" + expert2.getExpertID();
                String reversePairKey = expert2.getExpertID() + "-" + expert1.getExpertID();

                boolean isNewPair = !currentPairs.contains(pairKey) && !currentPairs.contains(reversePairKey);

                if (isNewPair && isPairAllowedInPreviousRounds(previousRounds, expert1, expert2)) {
                    currentPairs.add(pairKey);
                    currentRound.add(Arrays.asList(expert1, expert2));
                    index += 2;
                } else {
                    Collections.shuffle(experts);
                    index = 0;
                    currentRound.clear();
                    currentPairs.clear();
                }
            }

            previousRounds.add(currentPairs);

            // 将当前轮次及分组存入Map，并加入到最终的结果列表中
            Map<String, Object> roundMap = new HashMap<>();
            roundMap.put("round", "round" + (i + 1)); // 标记轮次
            roundMap.put("debates", currentRound); // 存入分组数据
            assignedExperts.add(roundMap);
        }

        return assignedExperts;
    }

    private boolean isPairAllowedInPreviousRounds(List<Set<String>> previousRounds, ExpertMesVO expert1, ExpertMesVO expert2) {
        String pairKey = expert1.getExpertID() + "-" + expert2.getExpertID();
        String reversePairKey = expert2.getExpertID() + "-" + expert1.getExpertID();

        for (Set<String> round : previousRounds) {
            if (round.contains(pairKey) || round.contains(reversePairKey)) {
                return false;
            }
        }
        return true;
    }
//    public static void main(String[] args) {
//        Z_ExpertAssign z = new Z_ExpertAssign();
//        List<List<List<ExpertMesVO>>> result = z.assignExperts(expertController.queryAllExpert(expertMesVO));
//        System.out.println(result); // 输出总共六组的分组
//        // 格式化输出每一轮的分组情况
//        for (int i = 0; i < result.size(); i++) {
//            System.out.println("Round " + (i + 1) + ":");
//            for (List<ExpertMesVO> group : result.get(i)) {
//                System.out.println(group);
//            }
//        }
//    }

}
